@@ -29,8 +29,6 @@
 #include <gmodule.h>
 #include "imsettings-module.h"
 
-#define IMSETTINGS_MODULE_GET_PRIVATE(_o_)	(G_TYPE_INSTANCE_GET_PRIVATE ((_o_), IMSETTINGS_TYPE_MODULE, IMSettingsModulePrivate))
-
 
 /**
  * SECTION:imsettings-module
@@ -54,7 +52,7 @@ struct _IMSettingsModulePrivate {
 	gpointer  dump_im;
 };
 
-G_DEFINE_TYPE (IMSettingsModule, imsettings_module, G_TYPE_OBJECT);
+G_DEFINE_TYPE_WITH_PRIVATE (IMSettingsModule, imsettings_module, G_TYPE_OBJECT);
 
 /*< private >*/
 static void
@@ -121,8 +119,6 @@ imsettings_module_class_init(IMSettingsModuleClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-	g_type_class_add_private(klass, sizeof (IMSettingsModulePrivate));
-
 	object_class->set_property = imsettings_module_set_property;
 	object_class->get_property = imsettings_module_get_property;
 	object_class->finalize     = imsettings_module_finalize;
@@ -139,7 +135,7 @@ imsettings_module_class_init(IMSettingsModuleClass *klass)
 static void
 imsettings_module_init(IMSettingsModule *module)
 {
-	module->priv = IMSETTINGS_MODULE_GET_PRIVATE (module);
+	module->priv = imsettings_module_get_instance_private (module);
 }
 
 /*< public >*/
