@@ -33,8 +33,6 @@
 #include "imsettings-marshal.h"
 #include "imsettings-proc.h"
 
-#define IMSETTINGS_PROC_GET_PRIVATE(_o_)	(G_TYPE_INSTANCE_GET_PRIVATE ((_o_), IMSETTINGS_TYPE_PROC, IMSettingsProcPrivate))
-
 #define RESTART_COUNTER_THRESHOLD 5
 #define MAXRESTART 3
 
@@ -84,7 +82,7 @@ static gboolean _stop_process                (IMSettingsProc     *proc,
                                               const gchar        *prog_name,
                                               GError             **error);
 
-G_DEFINE_TYPE (IMSettingsProc, imsettings_proc, G_TYPE_OBJECT);
+G_DEFINE_TYPE_WITH_PRIVATE (IMSettingsProc, imsettings_proc, G_TYPE_OBJECT);
 
 guint signals[LAST_SIGNAL] = { 0 };
 
@@ -564,8 +562,6 @@ imsettings_proc_class_init(IMSettingsProcClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-	g_type_class_add_private(klass, sizeof (IMSettingsProcPrivate));
-
 	object_class->set_property = imsettings_proc_set_property;
 	object_class->get_property = imsettings_proc_get_property;
 	object_class->finalize = imsettings_proc_finalize;
@@ -600,7 +596,7 @@ imsettings_proc_class_init(IMSettingsProcClass *klass)
 static void
 imsettings_proc_init(IMSettingsProc *proc)
 {
-	proc->priv = IMSETTINGS_PROC_GET_PRIVATE (proc);
+	proc->priv = imsettings_proc_get_instance_private (proc);
 }
 
 /*< public >*/
