@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /* 
  * imsettings-server.c
- * Copyright (C) 2008-2012 Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2008-2016 Red Hat, Inc. All rights reserved.
  * 
  * Authors:
  *   Akira TAGOH  <tagoh@redhat.com>
@@ -38,8 +38,6 @@
 #include "imsettings-utils.h"
 #include "imsettings-server.h"
 
-#define IMSETTINGS_SERVER_GET_PRIVATE(_o_)	(G_TYPE_INSTANCE_GET_PRIVATE ((_o_), IMSETTINGS_TYPE_SERVER, IMSettingsServerPrivate))
-
 struct _IMSettingsServerPrivate {
 	GDBusConnection    *connection;
 	NotifyNotification *notify;
@@ -95,7 +93,7 @@ GDBusInterfaceVTable __iface_vtable = {
 };
 guint signals[LAST_SIGNAL];
 
-G_DEFINE_TYPE (IMSettingsServer, imsettings_server, G_TYPE_OBJECT);
+G_DEFINE_TYPE_WITH_PRIVATE (IMSettingsServer, imsettings_server, G_TYPE_OBJECT);
 G_LOCK_DEFINE_STATIC (logger);
 
 /*< private >*/
@@ -404,8 +402,6 @@ imsettings_server_class_init(IMSettingsServerClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-	g_type_class_add_private(klass, sizeof (IMSettingsServerPrivate));
-
 	object_class->set_property = imsettings_server_set_property;
 	object_class->get_property = imsettings_server_get_property;
 	object_class->finalize     = imsettings_server_finalize;
@@ -465,7 +461,7 @@ imsettings_server_init(IMSettingsServer *server)
 {
 	IMSettingsServerPrivate *priv;
 
-	priv = server->priv = IMSETTINGS_SERVER_GET_PRIVATE (server);
+	priv = server->priv = imsettings_server_get_instance_private (server);
 
 	priv->active = FALSE;
 	priv->logging = TRUE;
@@ -685,9 +681,71 @@ imsettings_server_cb_get_info_variant(IMSettingsServer  *server,
 	return value;
 }
 
+static gchar *
+_escape_desktop_name(const gchar *name)
+{
+	if (name) {
+		size_t l, len = strlen(name);
+		GString *ss = g_string_new(NULL);
+
+		for (l = 0; l < len; l++) {
+			if ((name[l] >= 'A' && name[l] <= 'Z') ||
+			    (name[l] >= 'a' && name[l] <= 'z') ||
+			    (name[l] >= '0' && name[l] <= '9') ||
+			    name[l] == '-' || name[l] == '+' || name[l] == '_' || name[l] == '.' || name[l] == ',')
+				g_string_append_c(ss, name[l]);
+			else
+				g_string_append_c(ss, '_');
+		}
+		return g_string_free(ss, FALSE);
+	}
+
+	return NULL;
+}
+
+static gboolean
+imsettings_server_cb_is_supported_desktop(IMSettingsServer  *server,
+					  const gchar       *desktop)
+{
+	const gchar *pp;
+	gchar *helper_path, *script, *quoted_desktop = NULL;
+	gchar *cmd;
+	gint ret;
+	gboolean match = FALSE;
+	GError *err = NULL;
+
+	quoted_desktop = _escape_desktop_name(desktop);
+	pp = g_getenv("IMSETTINGS_HELPER_PATH");
+	if (pp)
+		helper_path = g_strdup(pp);
+	else
+		helper_path = g_strdup(IMSETTINGS_HELPER_PATH);
+	script = g_build_filename(helper_path, "imsettings-target-checker.sh", NULL);
+	if (desktop)
+		cmd = g_strdup_printf("bash -c 'export XDG_CURRENT_DESKTOP=%s; bash %s'", quoted_desktop ? quoted_desktop : "", script);
+	else
+		cmd = g_strdup_printf("bash -c %s", script);
+	g_free(script);
+	g_free(helper_path);
+
+	if (g_spawn_command_line_sync(cmd, NULL, NULL, &ret, &err))
+		match = TRUE;
+	if (!g_spawn_check_exit_status(ret, &err))
+		match = FALSE;
+	if (err) {
+		g_warning("%s", err->message);
+		g_error_free(err);
+	}
+	g_free(cmd);
+	g_free(quoted_desktop);
+
+	return !match;
+}
+
 static gboolean
 imsettings_server_cb_switch_im(IMSettingsServer  *server,
 			       const gchar       *lang,
+			       const gchar       *desktop,
 			       const gchar       *module,
 			       gboolean           update_xinputrc,
 			       IMSettingsInfo   **info,
@@ -695,19 +753,15 @@ imsettings_server_cb_switch_im(IMSettingsServer  *server,
 {
 	IMSettingsServerPrivate *priv = server->priv;
 	GVariant *v;
-	gchar *conffile = NULL, *backupfile = NULL, *p = NULL, *n = NULL;
+	gchar *conffile = NULL, *backupfile = NULL, *p = NULL, *n = NULL, *quoted_desktop = NULL;
 	const gchar *homedir, *xinputfile;
 	struct stat st;
 	gint save_errno;
-	const gchar *pp;
-	gchar *helper_path, *script;
-	gboolean match = FALSE;
-	gchar *cmd;
-	gint ret;
 
+	quoted_desktop = _escape_desktop_name(desktop);
 	g_log(G_LOG_DOMAIN, G_LOG_LEVEL_INFO,
-	      "Attempting to switch IM to %s [lang=%s, update=%s]",
-	      module, lang, update_xinputrc ? "true" : "false");
+	      "Attempting to switch IM to %s [lang=%s, desktop=%s, update=%s]",
+	      module, lang, quoted_desktop, update_xinputrc ? "true" : "false");
 
 	if (g_ascii_strcasecmp(module, IMSETTINGS_NONE_CONF) == 0 ||
 	    g_ascii_strcasecmp(module, IMSETTINGS_NONE_CONF XINPUT_SUFFIX) == 0) {
@@ -737,24 +791,7 @@ imsettings_server_cb_switch_im(IMSettingsServer  *server,
 		return FALSE;
 	}
 
-	pp = g_getenv("IMSETTINGS_HELPER_PATH");
-	if (pp)
-		helper_path = g_strdup(pp);
-	else
-		helper_path = g_strdup(IMSETTINGS_HELPER_PATH);
-	script = g_build_filename(helper_path, "imsettings-target-checker.sh", NULL);
-	cmd = g_strdup_printf("sh %s", script);
-	g_free(script);
-	g_free(helper_path);
-
-	if (g_spawn_command_line_sync(cmd, NULL, NULL, &ret, error))
-		match = TRUE;
-	if (!g_spawn_check_exit_status(ret, error))
-		match = FALSE;
-	g_free(cmd);
-	if (*error) {
-		g_clear_error(error);
-	} else if (match) {
+	if (!imsettings_server_cb_is_supported_desktop(server, desktop)) {
 		g_set_error(error, IMSETTINGS_GERROR, IMSETTINGS_GERROR_NOT_TARGETED_DESKTOP,
 			    _("Current desktop isn't targeted by IMSettings."));
 		g_warning("%s", (*error)->message);
@@ -769,8 +806,10 @@ imsettings_server_cb_switch_im(IMSettingsServer  *server,
 		g_object_remove_weak_pointer(G_OBJECT (priv->current_im),
 					     (gpointer *)&priv->current_im);
 		imsettings_proc_kill(priv->current_im, error);
-		if (*error)
+		if (*error) {
+			g_free(quoted_desktop);
 			return FALSE;
+		}
 
 		/* do not unref priv->current_im here.
 		 * it makes defunct process and
@@ -785,6 +824,8 @@ imsettings_server_cb_switch_im(IMSettingsServer  *server,
 	g_signal_connect(priv->current_im, "notify_notification",
 			 G_CALLBACK (_notify_cb),
 			 server);
+	g_object_set(G_OBJECT (priv->current_im), "desktop", quoted_desktop, NULL);
+	g_free(quoted_desktop);
 
 	if (update_xinputrc) {
 		homedir = imsettings_server_get_homedir(server);
@@ -1039,9 +1080,11 @@ imsettings_server_bus_method_call(GDBusConnection       *connection,
 	d(g_print("%s: sender[%s] path[%s] iface[%s] method[%s]\n", __PRETTY_FUNCTION__, sender, object_path, interface_name, method_name));
 
 	if (g_strcmp0(method_name, "StopService") == 0) {
+		value = g_variant_new_boolean(TRUE);
 		g_dbus_method_invocation_return_value(invocation,
-						      g_variant_new_boolean(TRUE));
+						      g_variant_new_tuple(&value, 1));
 		g_signal_emit(server, signals[SIG_DISCONNECTED], 0, NULL);
+		value = NULL;
 	} else if (g_strcmp0(method_name, "GetVersion") == 0) {
 		value = g_variant_new_uint32(IMSETTINGS_SETTINGS_API_VERSION);
 	} else if (g_strcmp0(method_name, "GetInfoVariants") == 0) {
@@ -1146,7 +1189,31 @@ imsettings_server_bus_method_call(GDBusConnection       *connection,
 
 		g_variant_get(parameters, "(&s&sb)",
 			      &lang, &module, &update);
-		ret = imsettings_server_cb_switch_im(server, lang, module, update, &info, &err);
+		ret = imsettings_server_cb_switch_im(server, lang, NULL, module, update, &info, &err);
+		if (ret) {
+			GHashTableIter iter;
+			gpointer key, val;
+			IMSettingsModule *mod;
+
+			g_hash_table_iter_init(&iter, priv->modules);
+			while (g_hash_table_iter_next(&iter, &key, &val)) {
+				mod = IMSETTINGS_MODULE (val);
+				if (mod)
+					imsettings_module_switch_im(mod, info);
+			}
+		}
+		if (info)
+			g_object_unref(info);
+
+		value = g_variant_new_boolean(ret);
+	} else if (g_strcmp0(method_name, "SwitchIMNoDesktopDetection") == 0) {
+		const gchar *lang, *desktop, *module;
+		gboolean update, ret;
+		IMSettingsInfo *info = NULL;
+
+		g_variant_get(parameters, "(&s&s&sb)",
+			      &lang, &desktop, &module, &update);
+		ret = imsettings_server_cb_switch_im(server, lang, desktop, module, update, &info, &err);
 		if (ret) {
 			GHashTableIter iter;
 			gpointer key, val;
@@ -1206,6 +1273,14 @@ imsettings_server_bus_method_call(GDBusConnection       *connection,
 				    IMSETTINGS_GERROR_CONFIGURATION_ERROR,
 				    "No modules loaded");
 		}
+	} else if (g_strcmp0(method_name, "IsSupportedDesktop") == 0) {
+		const gchar *desktop;
+		gboolean ret;
+
+		g_variant_get(parameters, "(&s)",
+			      &desktop);
+		ret = imsettings_server_cb_is_supported_desktop(server, desktop);
+		value = g_variant_new_boolean(ret);
 	}
   finalize:
 	/* GVariant uses a floating reference count system.
