@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /* 
  * imsettings-client.c
- * Copyright (C) 2008-2012 Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2008-2015 Red Hat, Inc. All rights reserved.
  * 
  * Authors:
  *   Akira TAGOH  <tagoh@redhat.com>
@@ -25,13 +25,18 @@
 #include "config.h"
 #endif
 
+#include <locale.h>
 #include <glib/gi18n-lib.h>
 #include "imsettings.h"
 #include "imsettings-utils.h"
 #include "imsettings-client.h"
 
-#define IMSETTINGS_CLIENT_GET_PRIVATE(_o_)	(G_TYPE_INSTANCE_GET_PRIVATE ((_o_), IMSETTINGS_TYPE_CLIENT, IMSettingsClientPrivate))
 
+struct _IMSettingsClientPrivate {
+	GDBusProxy *proxy;
+	gchar      *locale;
+	gchar      *desktop;
+};
 
 /**
  * SECTION:imsettings-client
@@ -51,15 +56,12 @@
  * Please see documentation of each methods for more details of DBus call.
  */
 
-G_DEFINE_TYPE (IMSettingsClient, imsettings_client, G_TYPE_OBJECT);
+G_DEFINE_TYPE_WITH_PRIVATE (IMSettingsClient, imsettings_client, G_TYPE_OBJECT);
 
-struct _IMSettingsClientPrivate {
-	GDBusProxy *proxy;
-	gchar      *locale;
-};
 enum {
 	PROP_0,
 	PROP_LOCALE,
+	PROP_DESKTOP,
 	LAST_PROP
 };
 
@@ -107,6 +109,10 @@ imsettings_client_set_property(GObject      *object,
 		    imsettings_client_set_locale(IMSETTINGS_CLIENT (object),
 						 g_value_get_string(value));
 		    break;
+	    case PROP_DESKTOP:
+		    imsettings_client_set_desktop(IMSETTINGS_CLIENT (object),
+						  g_value_get_string(value));
+		    break;
 	    default:
 		    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		    break;
@@ -124,6 +130,10 @@ imsettings_client_get_property(GObject    *object,
 		    g_value_set_string(value,
 				       imsettings_client_get_locale(IMSETTINGS_CLIENT (object)));
 		    break;
+	    case PROP_DESKTOP:
+		    g_value_set_string(value,
+				       imsettings_client_get_desktop(IMSETTINGS_CLIENT (object)));
+		    break;
 	    default:
 		    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		    break;
@@ -139,6 +149,7 @@ imsettings_client_finalize(GObject *object)
 	if (priv->proxy)
 		g_object_unref(priv->proxy);
 	g_free(priv->locale);
+	g_free(priv->desktop);
 
 	if (G_OBJECT_CLASS (imsettings_client_parent_class)->finalize)
 		G_OBJECT_CLASS (imsettings_client_parent_class)->finalize(object);
@@ -149,8 +160,6 @@ imsettings_client_class_init(IMSettingsClientClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-	g_type_class_add_private(klass, sizeof (IMSettingsClientPrivate));
-
 	object_class->set_property = imsettings_client_set_property;
 	object_class->get_property = imsettings_client_get_property;
 	object_class->finalize     = imsettings_client_finalize;
@@ -162,6 +171,12 @@ imsettings_client_class_init(IMSettingsClientClass *klass)
 							    _("Locale to get the imsettings information"),
 							    NULL,
 							    G_PARAM_READWRITE));
+	g_object_class_install_property(object_class, PROP_DESKTOP,
+					g_param_spec_string("desktop",
+							    _("Desktop"),
+							    _("Current desktop"),
+							    NULL,
+							    G_PARAM_READWRITE));
 }
 
 static void
@@ -169,10 +184,11 @@ imsettings_client_init(IMSettingsClient *client)
 {
 	IMSettingsClientPrivate *priv;
 
-	priv = client->priv = IMSETTINGS_CLIENT_GET_PRIVATE (client);
+	priv = client->priv = imsettings_client_get_instance_private (client);
 
 	priv->proxy = NULL;
 	priv->locale = NULL;
+	priv->desktop = NULL;
 }
 
 G_INLINE_FUNC gboolean
@@ -322,6 +338,54 @@ imsettings_client_get_locale(IMSettingsClient *client)
 	return priv->locale;
 }
 
+/**
+ * imsettings_client_set_desktop:
+ * @client: a #IMSettingsClient.
+ * @desktop: a current desktop name going to operate.
+ *
+ * Update the desktop name in @client to @desktop. if @desktop is %NULL,
+ * the desktop detection will be done in the server side.
+ *
+ * Returns: %TRUE if successfully updated, otherwise %FALSE.
+ */
+gboolean
+imsettings_client_set_desktop(IMSettingsClient *client,
+			      const gchar      *desktop)
+{
+	IMSettingsClientPrivate *priv;
+
+	g_return_val_if_fail (IMSETTINGS_IS_CLIENT (client), FALSE);
+
+	priv = client->priv;
+	g_free(priv->desktop);
+	priv->desktop = g_strdup(desktop);
+
+	g_object_notify(G_OBJECT (client), "desktop");
+
+	return TRUE;
+}
+
+/**
+ * imsettings_client_get_desktop:
+ * @client: a #IMSettingsClient.
+ *
+ * Obtains current desktop information in @client.
+ *
+ * Returns: a reference to the desktop name in @client. it shouldn't be freed
+ *          in applications.
+ */
+const gchar *
+imsettings_client_get_desktop(IMSettingsClient *client)
+{
+	IMSettingsClientPrivate *priv;
+
+	g_return_val_if_fail (IMSETTINGS_IS_CLIENT (client), NULL);
+
+	priv = client->priv;
+
+	return priv->desktop;
+}
+
 /**
  * imsettings_client_get_version:
  * @client: a #IMSettingsClient.
@@ -884,6 +948,7 @@ imsettings_client_switch_im(IMSettingsClient  *client,
 	gboolean retval = FALSE;
 	GVariant *value;
 	gchar *m;
+	const gchar *desktop;
 
 	g_return_val_if_fail (IMSETTINGS_IS_CLIENT (client), FALSE);
 
@@ -891,17 +956,31 @@ imsettings_client_switch_im(IMSettingsClient  *client,
 		m = g_strdup(IMSETTINGS_NONE_CONF);
 	else
 		m = g_strdup(module);
+	desktop = imsettings_client_get_desktop(client);
 
 	proxy = imsettings_client_get_proxy(client);
-	value = g_dbus_proxy_call_sync(proxy,
-				       "SwitchIM",
-				       g_variant_new("(ssb)",
-						     imsettings_client_get_locale(client),
-						     m, update_xinputrc),
-				       G_DBUS_CALL_FLAGS_NONE,
-				       -1,
-				       cancellable,
-				       error);
+	if (desktop) {
+		value = g_dbus_proxy_call_sync(proxy,
+					       "SwitchIMNoDesktopDetection",
+					       g_variant_new("(sssb)",
+							     imsettings_client_get_locale(client),
+							     desktop,
+							     m, update_xinputrc),
+					       G_DBUS_CALL_FLAGS_NONE,
+					       -1,
+					       cancellable,
+					       error);
+	} else {
+		value = g_dbus_proxy_call_sync(proxy,
+					       "SwitchIM",
+					       g_variant_new("(ssb)",
+							     imsettings_client_get_locale(client),
+							     m, update_xinputrc),
+					       G_DBUS_CALL_FLAGS_NONE,
+					       -1,
+					       cancellable,
+					       error);
+	}
 	if (value != NULL) {
 		g_variant_get(value, "(b)", &retval);
 		g_variant_unref(value);
@@ -934,6 +1013,7 @@ imsettings_client_switch_im_start(IMSettingsClient    *client,
 {
 	GDBusProxy *proxy;
 	gchar *m;
+	const gchar *desktop;
 
 	g_return_if_fail (IMSETTINGS_IS_CLIENT (client));
 
@@ -941,18 +1021,33 @@ imsettings_client_switch_im_start(IMSettingsClient    *client,
 		m = g_strdup(IMSETTINGS_NONE_CONF);
 	else
 		m = g_strdup(module);
+	desktop = imsettings_client_get_desktop(client);
 
 	proxy = imsettings_client_get_proxy(client);
-	g_dbus_proxy_call(proxy,
-			  "SwitchIM",
-			  g_variant_new("(ssb)",
-					imsettings_client_get_locale(client),
-					m, update_xinputrc),
-			  G_DBUS_CALL_FLAGS_NONE,
-			  -1,
-			  cancellable,
-			  callback,
-			  user_data);
+	if (desktop) {
+		g_dbus_proxy_call(proxy,
+				  "SwitchIMNoDesktopDetection",
+				  g_variant_new("(sssb)",
+						imsettings_client_get_locale(client),
+						desktop,
+						m, update_xinputrc),
+				  G_DBUS_CALL_FLAGS_NONE,
+				  -1,
+				  cancellable,
+				  callback,
+				  user_data);
+	} else {
+		g_dbus_proxy_call(proxy,
+				  "SwitchIM",
+				  g_variant_new("(ssb)",
+						imsettings_client_get_locale(client),
+						m, update_xinputrc),
+				  G_DBUS_CALL_FLAGS_NONE,
+				  -1,
+				  cancellable,
+				  callback,
+				  user_data);
+	}
 	g_free(m);
 }
 
@@ -1372,3 +1467,106 @@ imsettings_client_ping(IMSettingsClient *client)
 
 	return FALSE;
 }
+
+/**
+ * imsettings_client_is_supported_desktop:
+ * @client: a #IMSettingsClient.
+ * @cancellable: (allow-none): a #GCancellable or %NULL.
+ * @error: (allow-none): a #GError to store an error if any, or %NULL.
+ *
+ * Check if current desktop is supported by IMSettings or not.
+ *
+ * You could access through DBus API instead:
+ *
+ * |[
+ *   &lt;interface name='com.redhat.imsettings'&gt;
+ *     &lt;method name='IsSupportedDesktop'&gt;
+ *       &lt;arg type='s' name='desktop' direction='in' /&gt;
+ *       &lt;arg type='b' name='ret' direction'out' /&gt;
+ *     &lt;/method&gt;
+ *   &lt;/interface&gt;
+ * ]|
+ *
+ * Returns: if supported, returns %TRUE otherwise %FALSE.
+ */
+gboolean
+imsettings_client_is_supported_desktop(IMSettingsClient  *client,
+				       GCancellable      *cancellable,
+				       GError           **error)
+{
+	const gchar *desktop;
+	GDBusProxy *proxy;
+	GVariant *value;
+	gboolean retval = FALSE;
+
+	g_return_val_if_fail (IMSETTINGS_IS_CLIENT (client), FALSE);
+
+	desktop = imsettings_client_get_desktop(client);
+	proxy = imsettings_client_get_proxy(client);
+	value = g_dbus_proxy_call_sync(proxy,
+				       "IsSupportedDesktop",
+				       g_variant_new("(s)",
+						     desktop ? desktop : ""),
+				       G_DBUS_CALL_FLAGS_NONE,
+				       -1,
+				       cancellable,
+				       error);
+	if (value) {
+		g_variant_get(value, "(b)", &retval);
+		g_variant_unref(value);
+	}
+
+	return retval;
+}
+
+/**
+ * imsettings_client_is_supported_desktop_start:
+ * @client: a #IMSettingsClient.
+ * @cancellable: (allow-none): a #GCancellable or %NULL.
+ * @callback: (scope async): a #GAsyncReadyCallback.
+ * @user_data: (closure): a pointer of the user data to give it to @callback.
+ *
+ * Request asking if current desktop is supported or not asynchronously
+ */
+void
+imsettings_client_is_supported_desktop_start(IMSettingsClient    *client,
+					     GCancellable        *cancellable,
+					     GAsyncReadyCallback  callback,
+					     gpointer             user_data)
+{
+	GDBusProxy *proxy;
+	const gchar *desktop;
+
+	g_return_if_fail (IMSETTINGS_IS_CLIENT (client));
+
+	desktop = imsettings_client_get_desktop(client);
+	proxy = imsettings_client_get_proxy(client);
+	g_dbus_proxy_call(proxy,
+			  "IsSupportedDesktop",
+			  g_variant_new("(s)", desktop ? desktop : ""),
+			  G_DBUS_CALL_FLAGS_NONE,
+			  -1,
+			  cancellable,
+			  callback,
+			  user_data);
+}
+
+/**
+ * imsettings_client_is_supported_desktop_finish:
+ * @client: a #IMSettingsClient.
+ * @result: a #GAsyncResult pushed through #GAsyncReadyCallback.
+ * @error: (allow-none): a #GError to store an error if any, or %NULL.
+ *
+ * Obtains the result of the request from
+ * imsettings_client_is_supported_desktop_start().
+ *
+ * Returns: if the operation is successfully done, returns %TRUE
+ *          otherwise %FALSE
+ */
+gboolean
+imsettings_client_is_supported_desktop_finish(IMSettingsClient  *client,
+					      GAsyncResult      *result,
+					      GError           **error)
+{
+	return imsettings_client_async_result_boolean(client, result, error);
+}
