@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /* 
  * imsettings-info.c
- * Copyright (C) 2008-2012 Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2008-2015 Red Hat, Inc. All rights reserved.
  * 
  * Authors:
  *   Akira TAGOH  <tagoh@redhat.com>
@@ -26,6 +26,7 @@
 #endif
 
 #include <ctype.h>
+#include <locale.h>
 #include <stdio.h>
 #include <sys/stat.h>
 #include <glib/gi18n-lib.h>
@@ -33,8 +34,6 @@
 #include "imsettings-client.h"
 #include "imsettings-info.h"
 
-#define IMSETTINGS_INFO_GET_PRIVATE(_o_)	(G_TYPE_INSTANCE_GET_PRIVATE ((_o_), IMSETTINGS_TYPE_INFO, IMSettingsInfoPrivate))
-
 
 /**
  * SECTION:imsettings-info
@@ -79,7 +78,7 @@ struct _IMSettingsInfoPrivate {
 
 static GQuark __xinput_tokens[LAST_IMSETTINGS_INFO] = { 0 };
 
-G_DEFINE_TYPE (IMSettingsInfo, imsettings_info, G_TYPE_OBJECT);
+G_DEFINE_TYPE_WITH_PRIVATE (IMSettingsInfo, imsettings_info, G_TYPE_OBJECT);
 G_LOCK_DEFINE_STATIC (info);
 
 /*< private >*/
@@ -240,8 +239,6 @@ imsettings_info_class_init(IMSettingsInfoClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-	g_type_class_add_private(klass, sizeof (IMSettingsInfoPrivate));
-
 	object_class->finalize = imsettings_info_finalize;
 
 	/* properties */
@@ -250,7 +247,7 @@ imsettings_info_class_init(IMSettingsInfoClass *klass)
 static void
 imsettings_info_init(IMSettingsInfo *info)
 {
-	info->priv = IMSETTINGS_INFO_GET_PRIVATE (info);
+	info->priv = imsettings_info_get_instance_private (info);
 	info->priv->info_table = g_hash_table_new_full(g_direct_hash,
 						       g_direct_equal,
 						       NULL,
